local lg = love.graphics
local lm = love.math
function love.load()
  font = love.graphics.newFont(20)
  love.window.setMode(1280, 720)

  --[[
    first create the canvas that a single
    circle will be drawn to
  --]]
  local cw = 100 -- canvas width
  local ch = 100 -- canvas height
  circleCanvas = lg.newCanvas(cw,ch)
  lg.setCanvas(circleCanvas)
    circleCanvas:clear()
    lg.setColor(255,255,255,100)
    lg.circle('fill', cw/2, ch/2, (cw/2) - 1)
    lg.setColor(255,255,255,255)
    lg.circle('line', cw/2, ch/2, (cw/2) -1 )
  lg.setCanvas()

  -- next, get the image data from the canvas
  circleImageData = circleCanvas:getImageData()

  --  now create an image from the ImageData object
  circleImg = lg.newImage(circleImageData)

  --  finally add the image to a new SpriteBatch
  sb = lg.newSpriteBatch(circleImg, 5000, 'stream')


  --[[
    To test the Spritebatch, we are going to create
    x# of circles with the for loop below, and store
    each circle in the circles table.  This will
    be used in the love.update() to move them
  ]]
  circles = {}

  --[[
   Adjust this value to test different number of circles
  and how it will affect the overall FPS
  --]]
  numCircles = 100

  --  Create the circles and add them to the SpriteBatch
  for i=1, numCircles, 1 do
    local circle = {}
    circle.x = lm.random(0, lg.getWidth())
    circle.y = lm.random(0, lg.getHeight())
    circle.radius = lm.random(10, 50)
    circle.velocity = {
      x = lm.random(-20, 20),
      y = lm.random(-20, 20)
    }

    circle.ox = circleImg:getWidth()/2
    circle.oy = circleImg:getHeight()/2
    circle.sx = ((circle.radius * 2)/circleImg:getWidth())
    circle.sy = ((circle.radius * 2)/circleImg:getHeight())
    circle.rotation = 0

    sb:bind()
      sb:setColor(lm.random(0, 255), lm.random(0, 255), lm.random(0, 255))
      circle.id = sb:add(circle.x, circle.y, circle.rotation, circle.sx, circle.sy, circle.ox, circle.oy, 0, 0)
    sb:unbind()

    table.insert(circles, circle)
  end


end

function love.update(dt)
  --  Update the x & y of each circle
  for i, circle in ipairs(circles) do
    circle.x = circle.x + (circle.velocity.x * dt)
    circle.y = circle.y + (circle.velocity.y * dt)
    sb:set(circle.id, circle.x, circle.y, circle.rotation, circle.sx, circle.sy, circle.ox, circle.oy, 0, 0)
  end
end

function love.draw()
  love.graphics.setFont(font)
  lg.draw(sb)
  lg.print('FPS: ' .. love.timer.getFPS(), 0, 0)

end
